<%inherit file="content.html" />

<p>At least one of your prints is ready for pickup. Please pay using the link below.</p>
<p>Attention! Not all attachments have been printed. Please check the section "Print status" below to find out which ones failed. Refer to the FAQ for more information.</p>