<%inherit file="content.html" />

<p>Woops! It seems like something went wrong during printing of the attachments.</p>
<p>Check the section 'Print status' below for more information.</p>
<p>You can try once more by resending the attachments.</p>
<p>Was the print successful? Please pay using the link below.</p>